@@ -2,7 +2,7 @@
  *   nano.h  --  This file is part of GNU nano.                           *
  *                                                                        *
  *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
- *   Copyright (C) 2014, 2015, 2016 Benno Schulenberg                     *
+ *   Copyright (C) 2014-2017 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -128,10 +128,20 @@
 #undef ENABLE_MOUSE
 #endif
 
-#if defined(DISABLE_WRAPPING) && defined(DISABLE_JUSTIFY)
-#define DISABLE_WRAPJUSTIFY 1
+#if defined(ENABLE_WRAPPING) || defined(ENABLE_JUSTIFY)
+#define ENABLED_WRAPORJUSTIFY 1
 #endif
 
+#define BACKWARD FALSE
+#define FORWARD TRUE
+
+#define BLIND FALSE
+#define VISIBLE TRUE
+
+#define JUSTFIND  0
+#define REPLACING 1
+#define INREGION  2
+
 /* Enumeration types. */
 typedef enum {
     NIX_FILE, DOS_FILE, MAC_FILE
@@ -149,19 +159,16 @@ typedef enum {
     SOFTMARK, HARDMARK
 } mark_type;
 
-typedef enum {
-    UPWARD, DOWNWARD
-} scroll_dir;
-
 typedef enum {
     CENTERING, FLOWING, STATIONARY
 } update_type;
 
 typedef enum {
     ADD, DEL, BACK, CUT, CUT_TO_EOF, REPLACE,
-#ifndef DISABLE_WRAPPING
+#ifdef ENABLE_WRAPPING
     SPLIT_BEGIN, SPLIT_END,
 #endif
+    INDENT, UNINDENT,
 #ifdef ENABLE_COMMENT
     COMMENT, UNCOMMENT, PREFLIGHT,
 #endif
@@ -169,7 +176,7 @@ typedef enum {
 } undo_type;
 
 /* Structure types. */
-#ifndef DISABLE_COLOR
+#ifdef ENABLE_COLOR
 typedef struct colortype {
     short fg;
 	/* This syntax's foreground color. */
@@ -258,7 +265,7 @@ typedef struct lintstruct {
 	/* Regex starts and ends within this line. */
 #define CWOULDBE	(1<<6)
 	/* An unpaired start match on or before this line. */
-#endif /* !DISABLE_COLOR */
+#endif /* ENABLE_COLOR */
 
 /* More structure types. */
 typedef struct filestruct {
@@ -270,7 +277,7 @@ typedef struct filestruct {
 	/* Next node. */
     struct filestruct *prev;
 	/* Previous node. */
-#ifndef DISABLE_COLOR
+#ifdef ENABLE_COLOR
     short *multidata;
 	/* Array of which multi-line regexes apply to this line. */
 #endif
@@ -300,6 +307,8 @@ typedef struct undo_group {
 	/* First line of group. */
     ssize_t bottom_line;
 	/* Last line of group. */
+    char **indentations;
+	/* String data used to restore the affected lines; one per line. */
     struct undo_group *next;
 } undo_group;
 
@@ -334,7 +343,7 @@ typedef struct undo {
 } undo;
 #endif /* !NANO_TINY */
 
-#ifndef DISABLE_HISTORIES
+#ifdef ENABLE_HISTORIES
 typedef struct poshiststruct {
     char *filename;
 	/* The file. */
@@ -392,7 +401,7 @@ typedef struct openfilestruct {
     char *lock_filename;
 	/* The path of the lockfile, if we created one. */
 #endif
-#ifndef DISABLE_COLOR
+#ifdef ENABLE_COLOR
     syntaxtype *syntax;
 	/* The  syntax struct for this file, if any. */
     colortype *colorstrings;
@@ -469,6 +478,7 @@ enum
 {
     TITLE_BAR = 0,
     LINE_NUMBER,
+    SELECTED_TEXT,
     STATUS_BAR,
     KEY_COMBO,
     FUNCTION_TAG,
@@ -559,22 +569,26 @@ enum
 #define CONTROL_RIGHT 0x402
 #define CONTROL_UP 0x403
 #define CONTROL_DOWN 0x404
-#define CONTROL_HOME 0x411
-#define CONTROL_END 0x412
-#define SHIFT_CONTROL_LEFT 0x405
-#define SHIFT_CONTROL_RIGHT 0x406
-#define SHIFT_CONTROL_UP 0x407
-#define SHIFT_CONTROL_DOWN 0x408
-#define SHIFT_CONTROL_HOME 0x413
-#define SHIFT_CONTROL_END 0x414
-#define SHIFT_ALT_LEFT 0x409
-#define SHIFT_ALT_RIGHT 0x40a
-#define SHIFT_ALT_UP 0x40b
-#define SHIFT_ALT_DOWN 0x40c
-#define SHIFT_PAGEUP 0x40d
-#define SHIFT_PAGEDOWN 0x40e
-#define SHIFT_HOME 0x40f
-#define SHIFT_END 0x410
+#define CONTROL_HOME 0x405
+#define CONTROL_END 0x406
+#define SHIFT_CONTROL_LEFT 0x411
+#define SHIFT_CONTROL_RIGHT 0x412
+#define SHIFT_CONTROL_UP 0x413
+#define SHIFT_CONTROL_DOWN 0x414
+#define SHIFT_CONTROL_HOME 0x415
+#define SHIFT_CONTROL_END 0x416
+#define ALT_LEFT 0x421
+#define ALT_RIGHT 0x422
+#define ALT_UP 0x423
+#define ALT_DOWN 0x424
+#define SHIFT_ALT_LEFT 0x431
+#define SHIFT_ALT_RIGHT 0x432
+#define SHIFT_ALT_UP 0x433
+#define SHIFT_ALT_DOWN 0x434
+#define SHIFT_HOME 0x455
+#define SHIFT_END 0x456
+#define SHIFT_PAGEUP 0x457
+#define SHIFT_PAGEDOWN 0x458
 
 #ifdef USE_SLANG
 #ifdef ENABLE_UTF8
